
from fopen import *
from Spectrum import Spectrum

class MGFFileReader:
    def __init__(self,f):
        self.h = fopen(f,'r')
        self.title = None
        self.use_global_charge = True
        self.have_global_charge = False
        for l in self.h:
            l = l.strip()
            if l.startswith('COM='):
                self.title = l[4:]
            if l.startswith('CHARGE='):
                chargestr = l[8:].rstrip('+')
                try:
                    self.charge = int(chargestr)
                    self.have_global_charge = True
                except:
                    self.charge = None
            if l.startswith('BEGIN IONS'):
                break
        
    def __iter__(self):
        return self.next()

    def next(self):
        s = None
        for l in self.h:
            l = l.strip()
            sl = l.split('=',1)
            if len(sl) == 2:
                if sl[0] == 'PEPMASS':
                    if not s:
                        s = Spectrum()
                    precursor = float(sl[1])
                    s.set('precursorMz',precursor)
                elif sl[0] == 'CHARGE':
                    self.use_global_charge = False
                    if not s:
                        s = Spectrum()
                    try:
                        chargestr = sl[1].rstrip('+')
                        s.set('precursorMz.precursorCharge',int(chargestr))
                    except:
                        pass
                elif sl[0] == 'TITLE':
                    if not s:
                        s = Spectrum()
                    s.set('title',sl[1])
            elif l.startswith('END IONS'):
                self.use_global_charge = True
            elif l.startswith('BEGIN IONS'):
                if self.use_global_charge and \
                       self.have_global_charge: 
                    s.set('precursorMz.precursorCharge',self.charge)
                s.set('msLevel',2)
                yield s        
                s = None
            else:
                try:
                    (mz,it) = map(float,l.split())
                except:
                    continue
                s.addPeak(mz,it)
        if s:
            if self.use_global_charge and \
                   self.have_global_charge:
                s.set('precursorMz.precursorCharge',self.charge)
	    s.set('msLevel',2)
            yield s
        self.h.close()

class MGFFileWriter:
    def __init__(self,s,f):
        self.h = fopen(f,'w')
        self.s = s

    def write(self,title=None,charge=None,spectitle=None):
        if title:
            self.h.write("COM=%s\n"%(title,))
        if charge:
            self.h.write("CHARGE=%s\n"%(charge,))
        for s in self.s:
            self.h.write("BEGIN IONS\n")
            if spectitle:
                self.h.write("TITLE=%s\n"%(spectitle%s.metadata(),))
            self.h.write("PEPMASS=%f\n"%(s.get('precursorMz',None),))
            if s.get('precursorMz.precursorCharge',None):
                self.h.write("CHARGE=%d+\n"%(s.get('precursorMz.precursorCharge',None),))
            for (mz,it) in zip(s.mz(),s.it()):
                self.h.write("%f %f\n"%(mz,it))
            self.h.write("END IONS\n")
            
        
                
