
from fopen import *
from Spectrum import *
from elementtree import ElementTree as ET
import re
from base64 import b64encode,b64decode
from array import array
import sys

class MzDataFileReader:

    def __init__(self,f):
        self.handle = fopen(f,'r')
        self.context = []
        self.tagre = re.compile(r'^{(.*)}(.*)$')
        self.ns = ''
        self.typemap = {}
        for tag in ('precursorMz',
                    'lowMz',
                    'highMz',
                    'totIonCurrent',
                    'basePeakMz',
                    'basePeakIntensity',
                    'startMz',
                    'endMz',
                    'precursorMz.precursorIntensity',
                    'precursorMz.windowWideness',
                    'precursorMz'
                    ):
            self.typemap[tag] = float
        for tag in ('num',
                    'msLevel',
                    'peaksCount',
                    'scanOrigin.num',
                    'precursorMz.precursorScanNum',
                    'precursorMz.precursorCharge',
                    'maldi.laserShootCount',
                    'maldi.laserIntensity',
                    ):
            self.typemap[tag] = int
        for tag in ('retentionTime','maldi.laserFrequency'):
            self.typemap[tag] = lambda s: float(s[2:-1])
        for tag in ('maldi.collisionGas',):
            self.typemap[tag] = lambda s: (s.lower() == 'true' or s != '0')

    def __iter__(self):
        return self.next()

    def make_path(self,*l):
        return '/'.join(map(lambda t: self.ns+t,l))

    def extract_scan(self,ele):

        scan = Spectrum()

        scan.set('num',int(ele.attrib['id']))

        specInst=None
        for e in ele.findall(self.make_path('spectrumDesc','spectrumSettings','spectrumInstrument')):
            specInst = e
            break

        scan.set('msLevel',int(specInst.attrib['msLevel']))
        for e in specInst.findall(self.make_path('cvParam')):
            if e.attrib['accession'] == 'PSI:XXXXXXXX':
                scan.set('retentionTime',float(e.attrib['value']))

        prec = None
        for e in ele.findall(self.make_path('spectrumDesc','precursorList','precursor')):
            prec = e
            break

        if prec:
            scan.set('precursorMz.precursorScanNum',int(prec.attrib['spectrumRef']))

            for e in prec.findall(self.make_path('*','cvParam')):
                if e.attrib['accession'] == 'PSI:1000040':
                    scan.set('precursorMz',float(e.attrib['value']))
                elif e.attrib['accession'] == 'PSI:1000041':
                    scan.set('precursorMz.precursorCharge',int(e.attrib['value']))
                elif e.attrib['accession'] == 'PSI:1000042':
                    scan.set('precursorMz.precursorIntensity',float(e.attrib['value']))

        for mzde in ele.findall(self.make_path('mzArrayBinary','data')):
            if mzde.attrib['precision'] == '64':
                mzdata = array('d',b64decode(''.join(mzde.text.split())))
            elif mzde.attrib['precision'] == '32':
                mzdata = array('f',b64decode(''.join(mzde.text.split())))
            else:
                raise "Bad precision specification"
            if mzde.attrib['endian'] != sys.byteorder:
                mzdata.byteswap()
            scan.mz(mzdata)
            break

        for itde in ele.findall(self.make_path('intenArrayBinary','data')):
            if itde.attrib['precision'] == '64':
                itdata = array('d',b64decode(''.join(itde.text.split())))
            elif itde.attrib['precision'] == '32':
                itdata = array('f',b64decode(''.join(itde.text.split())))
            else:
                raise "Bad precision specification"
            if itde.attrib['endian'] != sys.byteorder:
                itdata.byteswap()
            scan.it(itdata)
            break

        return scan

    def next(self):
        for (event, ele) in ET.iterparse(self.handle,('start','end')):
            # print event,self.context,ele.tag,ele.attrib

            if event == 'start':
                m = self.tagre.search(ele.tag)
                if m:
                    self.context.append(m.group(2))
                else:
                    self.context.append(ele.tag)
                if self.context == ['mzData']:
                    for (k,v) in ele.attrib.iteritems():
                        if k.endswith('xsi:schemaLocation'):
                            self.ns = '{%s}'%(v.split()[0],)
                continue

            if self.context[-2:] == ['spectrumList','spectrum']:

                yield self.extract_scan(ele)

                ele.clear()

            self.context.pop()

    
        

